@@ -37,7 +37,9 @@
 #include "gdata-parser.h"
 #include "gdata-gd-where.h"
 #include "gdata-private.h"
+#include "gdata-comparable.h"
 
+static void gdata_gd_organization_comparable_init (GDataComparableIface *iface);
 static void gdata_gd_organization_dispose (GObject *object);
 static void gdata_gd_organization_finalize (GObject *object);
 static void gdata_gd_organization_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec);
@@ -72,8 +74,8 @@ enum {
 	PROP_LOCATION
 };
 
-G_DEFINE_TYPE (GDataGDOrganization, gdata_gd_organization, GDATA_TYPE_PARSABLE)
-#define GDATA_GD_ORGANIZATION_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GDATA_TYPE_GD_ORGANIZATION, GDataGDOrganizationPrivate))
+G_DEFINE_TYPE_WITH_CODE (GDataGDOrganization, gdata_gd_organization, GDATA_TYPE_PARSABLE,
+                         G_IMPLEMENT_INTERFACE (GDATA_TYPE_COMPARABLE, gdata_gd_organization_comparable_init))
 
 static void
 gdata_gd_organization_class_init (GDataGDOrganizationClass *klass)
@@ -107,10 +109,10 @@ gdata_gd_organization_class_init (GDataGDOrganizationClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_NAME,
-				g_param_spec_string ("name",
-					"Name", "The name of the organization.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("name",
+	                                                      "Name", "The name of the organization.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataGDOrganization:title:
@@ -123,15 +125,15 @@ gdata_gd_organization_class_init (GDataGDOrganizationClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_TITLE,
-				g_param_spec_string ("title",
-					"Title", "The title of a person within the organization.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("title",
+	                                                      "Title", "The title of a person within the organization.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataGDOrganization:relation-type:
 	 *
-	 * A programmatic value that identifies the type of organization.
+	 * A programmatic value that identifies the type of organization. For example: %GDATA_GD_ORGANIZATION_WORK or %GDATA_GD_ORGANIZATION_OTHER.
 	 *
 	 * For more information, see the
 	 * <ulink type="http" url="http://code.google.com/apis/gdata/docs/2.0/elements.html#gdOrganization">GData specification</ulink>.
@@ -139,10 +141,10 @@ gdata_gd_organization_class_init (GDataGDOrganizationClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_RELATION_TYPE,
-				g_param_spec_string ("relation-type",
-					"Relation type", "A programmatic value that identifies the type of organization.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("relation-type",
+	                                                      "Relation type", "A programmatic value that identifies the type of organization.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataGDOrganization:label:
@@ -156,10 +158,10 @@ gdata_gd_organization_class_init (GDataGDOrganizationClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_LABEL,
-				g_param_spec_string ("label",
-					"Label", "A simple string value used to name this organization.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("label",
+	                                                      "Label", "A simple string value used to name this organization.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataGDOrganization:is-primary:
@@ -172,10 +174,10 @@ gdata_gd_organization_class_init (GDataGDOrganizationClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_IS_PRIMARY,
-				g_param_spec_boolean ("is-primary",
-					"Primary?", "Indicates which organization out of a group is primary.",
-					FALSE,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_boolean ("is-primary",
+	                                                       "Primary?", "Indicates which organization out of a group is primary.",
+	                                                       FALSE,
+	                                                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataGDOrganization:department:
@@ -188,10 +190,10 @@ gdata_gd_organization_class_init (GDataGDOrganizationClass *klass)
 	 * Since: 0.5.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_DEPARTMENT,
-				g_param_spec_string ("department",
-					"Department", "Specifies a department within the organization.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("department",
+	                                                      "Department", "Specifies a department within the organization.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataGDOrganization:job-description:
@@ -204,10 +206,10 @@ gdata_gd_organization_class_init (GDataGDOrganizationClass *klass)
 	 * Since: 0.5.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_JOB_DESCRIPTION,
-				g_param_spec_string ("job-description",
-					"Job description", "Description of a job within the organization.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("job-description",
+	                                                      "Job description", "Description of a job within the organization.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataGDOrganization:symbol:
@@ -220,10 +222,10 @@ gdata_gd_organization_class_init (GDataGDOrganizationClass *klass)
 	 * Since: 0.5.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_SYMBOL,
-				g_param_spec_string ("symbol",
-					"Symbol", "Symbol of the organization.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("symbol",
+	                                                      "Symbol", "Symbol of the organization.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataGDOrganization:location:
@@ -236,10 +238,26 @@ gdata_gd_organization_class_init (GDataGDOrganizationClass *klass)
 	 * Since: 0.6.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_LOCATION,
-				g_param_spec_object ("location",
-					"Location", "A place associated with the organization, e.g. office location.",
-					GDATA_TYPE_GD_WHERE,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_object ("location",
+	                                                      "Location", "A place associated with the organization, e.g. office location.",
+	                                                      GDATA_TYPE_GD_WHERE,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+}
+
+static gint
+compare_with (GDataComparable *self, GDataComparable *other)
+{
+	GDataGDOrganizationPrivate *a = ((GDataGDOrganization*) self)->priv, *b = ((GDataGDOrganization*) other)->priv;
+
+	if (g_strcmp0 (a->name, b->name) == 0 && g_strcmp0 (a->title, b->title) == 0 && g_strcmp0 (a->department, b->department) == 0)
+		return 0;
+	return 1;
+}
+
+static void
+gdata_gd_organization_comparable_init (GDataComparableIface *iface)
+{
+	iface->compare_with = compare_with;
 }
 
 static void
@@ -360,30 +378,20 @@ gdata_gd_organization_set_property (GObject *object, guint property_id, const GV
 static gboolean
 pre_parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *root_node, gpointer user_data, GError **error)
 {
-	xmlChar *rel, *primary;
+	xmlChar *rel;
 	gboolean primary_bool;
 	GDataGDOrganizationPrivate *priv = GDATA_GD_ORGANIZATION (parsable)->priv;
 
+	/* Is it the primary organisation? */
+	if (gdata_parser_boolean_from_property (root_node, "primary", &primary_bool, 0, error) == FALSE)
+		return FALSE;
+
 	rel = xmlGetProp (root_node, (xmlChar*) "rel");
 	if (rel != NULL && *rel == '\0') {
 		xmlFree (rel);
 		return gdata_parser_error_required_property_missing (root_node, "rel", error);
 	}
 
-	/* Is it the primary organisation? */
-	primary = xmlGetProp (root_node, (xmlChar*) "primary");
-	if (primary == NULL || xmlStrcmp (primary, (xmlChar*) "false") == 0)
-		primary_bool = FALSE;
-	else if (xmlStrcmp (primary, (xmlChar*) "true") == 0)
-		primary_bool = TRUE;
-	else {
-		gdata_parser_error_unknown_property_value (root_node, "primary", (gchar*) primary, error);
-		xmlFree (primary);
-		xmlFree (rel);
-		return FALSE;
-	}
-	xmlFree (primary);
-
 	priv->relation_type = (gchar*) rel;
 	priv->label = (gchar*) xmlGetProp (root_node, (xmlChar*) "label");
 	priv->is_primary = primary_bool;
@@ -394,53 +402,21 @@ pre_parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *root_node, gpointe
 static gboolean
 parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_data, GError **error)
 {
+	gboolean success;
 	GDataGDOrganizationPrivate *priv = GDATA_GD_ORGANIZATION (parsable)->priv;
 
-	if (gdata_parser_is_namespace (node, "http://schemas.google.com/g/2005") == FALSE)
-		return GDATA_PARSABLE_CLASS (gdata_gd_organization_parent_class)->parse_xml (parsable, doc, node, user_data, error);
-
-	if (xmlStrcmp (node->name, (xmlChar*) "orgName") == 0) {
-		/* gd:orgName */
-		if (priv->name != NULL)
-			return gdata_parser_error_duplicate_element (node, error);
-		priv->name = (gchar*) xmlNodeListGetString (doc, node->children, TRUE);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "orgTitle") == 0) {
-		/* gd:orgTitle */
-		if (priv->title != NULL)
-			return gdata_parser_error_duplicate_element (node, error);
-		priv->title = (gchar*) xmlNodeListGetString (doc, node->children, TRUE);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "orgDepartment") == 0) {
-		/* gd:orgDepartment */
-		if (priv->department != NULL)
-			return gdata_parser_error_duplicate_element (node, error);
-		priv->department = (gchar*) xmlNodeListGetString (doc, node->children, TRUE);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "orgJobDescription") == 0) {
-		/* gd:orgJobDescription */
-		if (priv->job_description != NULL)
-			return gdata_parser_error_duplicate_element (node, error);
-		priv->job_description = (gchar*) xmlNodeListGetString (doc, node->children, TRUE);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "orgSymbol") == 0) {
-		/* gd:orgSymbol */
-		if (priv->symbol != NULL)
-			return gdata_parser_error_duplicate_element (node, error);
-		priv->symbol = (gchar*) xmlNodeListGetString (doc, node->children, TRUE);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "where") == 0) {
-		/* gd:where */
-		GDataGDWhere *location = GDATA_GD_WHERE (_gdata_parsable_new_from_xml_node (GDATA_TYPE_GD_WHERE, doc, node, NULL, error));
-		if (location == NULL)
-			return FALSE;
-
-		if (priv->location != NULL) {
-			g_object_unref (location);
-			return gdata_parser_error_duplicate_element (node, error);
-		}
-
-		priv->location = location;
+	if (gdata_parser_is_namespace (node, "http://schemas.google.com/g/2005") == TRUE && (
+	     gdata_parser_string_from_element (node, "orgName", P_NO_DUPES, &(priv->name), &success, error) == TRUE ||
+	     gdata_parser_string_from_element (node, "orgTitle", P_NO_DUPES, &(priv->title), &success, error) == TRUE ||
+	     gdata_parser_string_from_element (node, "orgDepartment", P_NO_DUPES, &(priv->department), &success, error) == TRUE ||
+	     gdata_parser_string_from_element (node, "orgJobDescription", P_NO_DUPES, &(priv->job_description), &success, error) == TRUE ||
+	     gdata_parser_string_from_element (node, "orgSymbol", P_NO_DUPES, &(priv->symbol), &success, error) == TRUE ||
+	     gdata_parser_object_from_element (node, "where", P_REQUIRED | P_NO_DUPES, GDATA_TYPE_GD_WHERE,
+	                                      &(priv->location), &success, error) == TRUE)) {
+		return success;
 	} else {
 		return GDATA_PARSABLE_CLASS (gdata_gd_organization_parent_class)->parse_xml (parsable, doc, node, user_data, error);
 	}
-
-	return TRUE;
 }
 
 static void
@@ -504,39 +480,7 @@ gdata_gd_organization_new (const gchar *name, const gchar *title, const gchar *r
 {
 	g_return_val_if_fail (relation_type == NULL || *relation_type != '\0', NULL);
 	return g_object_new (GDATA_TYPE_GD_ORGANIZATION, "name", name, "title", title, "relation-type", relation_type,
-			     "label", label, "is-primary", is_primary, NULL);
-}
-
-/**
- * gdata_gd_organization_compare:
- * @a: a #GDataGDOrganization, or %NULL
- * @b: another #GDataGDOrganization, or %NULL
- *
- * Compares the two organizations in a strcmp() fashion. %NULL values are handled gracefully, with
- * <code class="literal">0</code> returned if both @a and @b are %NULL, <code class="literal">-1</code> if @a is %NULL
- * and <code class="literal">1</code> if @b is %NULL.
- *
- * The comparison of non-%NULL values is done on the basis of the @name property of the #GDataGDOrganization<!-- -->s.
- *
- * Return value: <code class="literal">0</code> if @a equals @b, <code class="literal">-1</code> or <code class="literal">1</code> as
- * appropriate otherwise
- *
- * Since: 0.4.0
- **/
-gint
-gdata_gd_organization_compare (const GDataGDOrganization *a, const GDataGDOrganization *b)
-{
-	if (a == NULL && b != NULL)
-		return -1;
-	else if (a != NULL && b == NULL)
-		return 1;
-
-	if (a == b)
-		return 0;
-	if (g_strcmp0 (a->priv->name, b->priv->name) == 0 && g_strcmp0 (a->priv->title, b->priv->title) == 0 &&
-	    g_strcmp0 (a->priv->department, b->priv->department) == 0)
-		return 0;
-	return 1;
+	                     "label", label, "is-primary", is_primary, NULL);
 }
 
 /**
