@@ -401,7 +401,7 @@ process_request (GDataAuthorizer *self, GDataAuthorizationDomain *domain, SoupMe
 			g_warning ("Not authorizing a non-HTTPS message with the user's ClientLogin auth token as the connection isn't secure.");
 		} else {
 			gchar *authorisation_header = g_strdup_printf ("GoogleLogin auth=%s", auth_token);
-			soup_message_headers_append (message->request_headers, "Authorization", authorisation_header);
+			soup_message_headers_replace (message->request_headers, "Authorization", authorisation_header);
 			g_free (authorisation_header);
 		}
 	}
@@ -489,7 +489,8 @@ gdata_client_login_authorizer_new_for_authorization_domains (const gchar *client
 	return authorizer;
 }
 
-/* Called in the main thread to notify of changes to the username and password properties from the authentication thread */
+/* Called in the main thread to notify of changes to the username and password properties from the authentication thread. It swallows a reference
+ * the authoriser. */
 static gboolean
 notify_authentication_details_cb (GDataClientLoginAuthorizer *self)
 {
@@ -500,6 +501,8 @@ notify_authentication_details_cb (GDataClientLoginAuthorizer *self)
 	g_object_notify (authorizer, "password");
 	g_object_thaw_notify (authorizer);
 
+	g_object_unref (self);
+
 	/* Only execute once */
 	return FALSE;
 }
@@ -530,9 +533,9 @@ set_authentication_details (GDataClientLoginAuthorizer *self, const gchar *usern
 	 *  • notifications will always be emitted before gdata_client_login_authorizer_authenticate() returns; and
 	 *  • notifications will always be emitted in the main thread for calls to gdata_client_login_authorizer_authenticate_async(). */
 	if (is_async == TRUE) {
-		g_idle_add ((GSourceFunc) notify_authentication_details_cb, self);
+		g_idle_add ((GSourceFunc) notify_authentication_details_cb, g_object_ref (self));
 	} else {
-		notify_authentication_details_cb (self);
+		notify_authentication_details_cb (g_object_ref (self));
 	}
 }
 
