@@ -493,7 +493,7 @@ pre_get_xml (GDataParsable *parsable, GString *xml_string)
 
 	/* Add the entry's ETag, if available */
 	if (gdata_entry_get_etag (GDATA_ENTRY (parsable)) != NULL)
-		g_string_append_printf (xml_string, " gd:etag='%s'", priv->etag);
+		gdata_parser_string_append_escaped (xml_string, " gd:etag='", priv->etag, "'");
 }
 
 static void
@@ -505,7 +505,7 @@ get_xml (GDataParsable *parsable, GString *xml_string)
 	gdata_parser_string_append_escaped (xml_string, "<title type='text'>", priv->title, "</title>");
 
 	if (priv->id != NULL)
-		g_string_append_printf (xml_string, "<id>%s</id>", priv->id);
+		gdata_parser_string_append_escaped (xml_string, "<id>", priv->id, "</id>");
 
 	if (priv->updated != -1) {
 		gchar *updated = gdata_parser_int64_to_iso8601 (priv->updated);
@@ -677,8 +677,18 @@ gdata_entry_set_summary (GDataEntry *self, const gchar *summary)
 const gchar *
 gdata_entry_get_id (GDataEntry *self)
 {
+	gchar *id;
+
 	g_return_val_if_fail (GDATA_IS_ENTRY (self), NULL);
-	return self->priv->id;
+
+	/* We have to get the actual property since GDataDocumentsEntry overrides it. We then store it in our own ID field so that we can
+	 * free it later on. */
+	g_object_get (G_OBJECT (self), "id", &id, NULL);
+
+	g_free (self->priv->id);
+	self->priv->id = id;
+
+	return id;
 }
 
 /**
