@@ -72,6 +72,10 @@ static void notify_timeout_cb (GObject *gobject, GParamSpec *pspec, GObject *sel
 static void debug_handler (const char *log_domain, GLogLevelFlags log_level, const char *message, gpointer user_data);
 static void soup_log_printer (SoupLogger *logger, SoupLoggerLogLevel level, char direction, const char *data, gpointer user_data);
 
+static GDataFeed *__gdata_service_query (GDataService *self, GDataAuthorizationDomain *domain, const gchar *feed_uri, GDataQuery *query,
+                                         GType entry_type, GCancellable *cancellable, GDataQueryProgressCallback progress_callback,
+                                         gpointer progress_user_data, GError **error, gboolean is_async);
+
 struct _GDataServicePrivate {
 	SoupSession *session;
 	gchar *locale;
@@ -280,6 +284,10 @@ real_append_query_headers (GDataService *self, GDataAuthorizationDomain *domain,
 	/* Set the authorisation header */
 	if (self->priv->authorizer != NULL) {
 		gdata_authorizer_process_request (self->priv->authorizer, domain, message);
+
+		/* Store the authorisation domain on the message so that we can access it again after refreshing authorisation if necessary.
+		 * See _gdata_service_send_message(). */
+		g_object_set_data_full (G_OBJECT (message), "gdata-authorization-domain", g_object_ref (domain), (GDestroyNotify) g_object_unref);
 	}
 
 	/* Set the GData-Version header to tell it we want to use the v2 API */
@@ -676,11 +684,22 @@ _gdata_service_send_message (GDataService *self, SoupMessage *message, GCancella
 	}
 
 	/* Not authorised, or authorisation has expired. If we were authorised in the first place, attempt to refresh the authorisation and
-	 * try sending the message again (but only once, so we don't get caught in an infinite loop of denied authorisation errors). */
+	 * try sending the message again (but only once, so we don't get caught in an infinite loop of denied authorisation errors).
+	 *
+	 * Note that we have to re-process the message with the authoriser so that its authorisation headers get updated after the refresh
+	 * (bgo#653535). */
 	if (message->status_code == SOUP_STATUS_UNAUTHORIZED) {
 		GDataAuthorizer *authorizer = self->priv->authorizer;
 
 		if (authorizer != NULL && gdata_authorizer_refresh_authorization (authorizer, cancellable, NULL) == TRUE) {
+			GDataAuthorizationDomain *domain;
+
+			/* Re-process the request */
+			domain = g_object_get_data (G_OBJECT (message), "gdata-authorization-domain");
+			g_assert (domain == NULL || GDATA_IS_AUTHORIZATION_DOMAIN (domain));
+
+			gdata_authorizer_process_request (authorizer, domain, message);
+
 			/* Send the message again */
 			g_clear_error (error);
 			_gdata_service_actually_send_message (self->priv->session, message, cancellable, error);
@@ -701,6 +720,7 @@ typedef struct {
 	GDataFeed *feed;
 	GDataQueryProgressCallback progress_callback;
 	gpointer progress_user_data;
+	GDestroyNotify destroy_progress_user_data;
 } QueryAsyncData;
 
 static void
@@ -725,16 +745,20 @@ query_thread (GSimpleAsyncResult *result, GDataService *service, GCancellable *c
 	QueryAsyncData *data = g_simple_async_result_get_op_res_gpointer (result);
 
 	/* Execute the query and return */
-	data->feed = gdata_service_query (service, data->domain, data->feed_uri, data->query, data->entry_type, cancellable,
-	                                  data->progress_callback, data->progress_user_data, &error);
+	data->feed = __gdata_service_query (service, data->domain, data->feed_uri, data->query, data->entry_type, cancellable,
+	                                    data->progress_callback, data->progress_user_data, &error, TRUE);
 	if (data->feed == NULL && error != NULL) {
 		g_simple_async_result_set_from_error (result, error);
 		g_error_free (error);
 	}
+
+	if (data->destroy_progress_user_data != NULL) {
+		data->destroy_progress_user_data (data->progress_user_data);
+	}
 }
 
 /**
- * gdata_service_query_async: (skip)
+ * gdata_service_query_async:
  * @self: a #GDataService
  * @domain: (allow-none): the #GDataAuthorizationDomain the query falls under, or %NULL
  * @feed_uri: the feed URI to query, including the host name and protocol
@@ -743,6 +767,8 @@ query_thread (GSimpleAsyncResult *result, GDataService *service, GCancellable *c
  * @cancellable: (allow-none): optional #GCancellable object, or %NULL
  * @progress_callback: (allow-none) (closure progress_user_data): a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
  * @progress_user_data: (closure): data to pass to the @progress_callback function
+ * @destroy_progress_user_data: (allow-none): the function to call when @progress_callback will not be called any more, or %NULL. This function will be
+ * called with @progress_user_data as a parameter and can be used to free any memory allocated for it.
  * @callback: a #GAsyncReadyCallback to call when the query is finished
  * @user_data: (closure): data to pass to the @callback function
  *
@@ -754,12 +780,12 @@ query_thread (GSimpleAsyncResult *result, GDataService *service, GCancellable *c
  * When the operation is finished, @callback will be called. You can then call gdata_service_query_finish()
  * to get the results of the operation.
  *
- * Since: 0.9.0
+ * Since: 0.9.1
  **/
 void
 gdata_service_query_async (GDataService *self, GDataAuthorizationDomain *domain, const gchar *feed_uri, GDataQuery *query, GType entry_type,
                            GCancellable *cancellable, GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
-                           GAsyncReadyCallback callback, gpointer user_data)
+                           GDestroyNotify destroy_progress_user_data, GAsyncReadyCallback callback, gpointer user_data)
 {
 	GSimpleAsyncResult *result;
 	QueryAsyncData *data;
@@ -778,6 +804,7 @@ gdata_service_query_async (GDataService *self, GDataAuthorizationDomain *domain,
 	data->entry_type = entry_type;
 	data->progress_callback = progress_callback;
 	data->progress_user_data = progress_user_data;
+	data->destroy_progress_user_data = destroy_progress_user_data;
 
 	result = g_simple_async_result_new (G_OBJECT (self), callback, user_data, gdata_service_query_async);
 	g_simple_async_result_set_op_res_gpointer (result, data, (GDestroyNotify) query_async_data_free);
@@ -859,6 +886,47 @@ _gdata_service_query (GDataService *self, GDataAuthorizationDomain *domain, cons
 	return message;
 }
 
+static GDataFeed *
+__gdata_service_query (GDataService *self, GDataAuthorizationDomain *domain, const gchar *feed_uri, GDataQuery *query, GType entry_type,
+                       GCancellable *cancellable, GDataQueryProgressCallback progress_callback, gpointer progress_user_data, GError **error,
+                       gboolean is_async)
+{
+	GDataServiceClass *klass;
+	GDataFeed *feed;
+	SoupMessage *message;
+
+	message = _gdata_service_query (self, domain, feed_uri, query, cancellable, error);
+	if (message == NULL)
+		return NULL;
+
+	g_assert (message->response_body->data != NULL);
+	klass = GDATA_SERVICE_GET_CLASS (self);
+	feed = _gdata_feed_new_from_xml (klass->feed_type, message->response_body->data, message->response_body->length, entry_type,
+	                                 progress_callback, progress_user_data, is_async, error);
+	g_object_unref (message);
+
+	if (feed == NULL)
+		return NULL;
+
+	/* Update the query with the feed's ETag */
+	if (query != NULL && feed != NULL && gdata_feed_get_etag (feed) != NULL)
+		gdata_query_set_etag (query, gdata_feed_get_etag (feed));
+
+	/* Update the query with the next and previous URIs from the feed */
+	if (query != NULL && feed != NULL) {
+		GDataLink *_link;
+
+		_link = gdata_feed_look_up_link (feed, "next");
+		if (_link != NULL)
+			_gdata_query_set_next_uri (query, gdata_link_get_uri (_link));
+		_link = gdata_feed_look_up_link (feed, "previous");
+		if (_link != NULL)
+			_gdata_query_set_previous_uri (query, gdata_link_get_uri (_link));
+	}
+
+	return feed;
+}
+
 /**
  * gdata_service_query:
  * @self: a #GDataService
@@ -899,10 +967,6 @@ GDataFeed *
 gdata_service_query (GDataService *self, GDataAuthorizationDomain *domain, const gchar *feed_uri, GDataQuery *query, GType entry_type,
                      GCancellable *cancellable, GDataQueryProgressCallback progress_callback, gpointer progress_user_data, GError **error)
 {
-	GDataServiceClass *klass;
-	GDataFeed *feed;
-	SoupMessage *message;
-
 	g_return_val_if_fail (GDATA_IS_SERVICE (self), NULL);
 	g_return_val_if_fail (domain == NULL || GDATA_IS_AUTHORIZATION_DOMAIN (domain), NULL);
 	g_return_val_if_fail (feed_uri != NULL, NULL);
@@ -910,36 +974,7 @@ gdata_service_query (GDataService *self, GDataAuthorizationDomain *domain, const
 	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), FALSE);
 	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-	message = _gdata_service_query (self, domain, feed_uri, query, cancellable, error);
-	if (message == NULL)
-		return NULL;
-
-	g_assert (message->response_body->data != NULL);
-	klass = GDATA_SERVICE_GET_CLASS (self);
-	feed = _gdata_feed_new_from_xml (klass->feed_type, message->response_body->data, message->response_body->length, entry_type,
-	                                 progress_callback, progress_user_data, error);
-	g_object_unref (message);
-
-	if (feed == NULL)
-		return NULL;
-
-	/* Update the query with the feed's ETag */
-	if (query != NULL && feed != NULL && gdata_feed_get_etag (feed) != NULL)
-		gdata_query_set_etag (query, gdata_feed_get_etag (feed));
-
-	/* Update the query with the next and previous URIs from the feed */
-	if (query != NULL && feed != NULL) {
-		GDataLink *_link;
-
-		_link = gdata_feed_look_up_link (feed, "next");
-		if (_link != NULL)
-			_gdata_query_set_next_uri (query, gdata_link_get_uri (_link));
-		_link = gdata_feed_look_up_link (feed, "previous");
-		if (_link != NULL)
-			_gdata_query_set_previous_uri (query, gdata_link_get_uri (_link));
-	}
-
-	return feed;
+	return __gdata_service_query (self, domain, feed_uri, query, entry_type, cancellable, progress_callback, progress_user_data, error, FALSE);
 }
 
 /**
