@@ -35,6 +35,10 @@
 #include <gdata/gdata-comparable.h>
 #include <gdata/gdata-batchable.h>
 #include <gdata/gdata-batch-operation.h>
+#include <gdata/gdata-authorizer.h>
+#include <gdata/gdata-authorization-domain.h>
+#include <gdata/gdata-client-login-authorizer.h>
+#include <gdata/gdata-oauth1-authorizer.h>
 
 /* Namespaces */
 
