@@ -29,6 +29,56 @@
  * For more details of Google PicasaWeb's GData API, see the <ulink type="http" url="http://code.google.com/apis/picasaweb/reference.html">
  * online documentation</ulink>.
  *
+ * <example>
+ * 	<title>Getting Basic Album Data</title>
+ * 	<programlisting>
+ *	GDataFeed *album_feed;
+ *	GList *album_entries;
+ *
+ *	/<!-- -->* Query for a feed of GDataPicasaWebAlbum<!-- -->s owned by user "libgdata.picasaweb" *<!-- -->/
+ *	album_feed = gdata_picasaweb_service_query_all_albums (service, NULL, "libgdata.picasaweb", NULL, NULL, NULL, NULL);
+ *
+ *	/<!-- -->* Get a list of GDataPicasaWebAlbum<!-- -->s from the query's feed *<!-- -->/
+ *	for (album_entries = gdata_feed_get_entries (album_feed); album_entries != NULL; album_entries = album_entries->next) {
+ *		GDataPicasaWebAlbum *album;
+ *		guint num_photos;
+ *		const gchar *owner_nickname, *title, *summary;
+ *		gint64 timestamp;
+ *		GList *thumbnails;
+ *
+ *		album = GDATA_PICASAWEB_ALBUM (album_entries->data);
+ *
+ *		/<!-- -->* Get various bits of information about the album *<!-- -->/
+ *		num_photos = gdata_picasaweb_album_get_num_photos (album);
+ *		owner_nickname = gdata_picasaweb_album_get_nickname (album);
+ *		title = gdata_entry_get_title (GDATA_ENTRY (album));
+ *		summary = gdata_entry_get_summary (GDATA_ENTRY (album));
+ *		/<!-- -->* Get the day the album was shot on or, if not set, when it was uploaded. This is in milliseconds since the epoch. *<!-- -->/
+ *		timestamp = gdata_picasaweb_album_get_timestamp (album);
+ *
+ *		for (thumbnails = gdata_picasaweb_album_get_thumbnails (album); thumbnails != NULL; thumbnails = thumbnails->next) {
+ *			GDataMediaThumbnail *thumbnail;
+ *			GDataDownloadStream *download_stream;
+ *			GdkPixbuf *pixbuf;
+ *
+ *			thumbnail = GDATA_MEDIA_THUMBNAIL (thumbnails->data);
+ *			/<!-- -->* Do something fun with the thumbnails, like download and display them. We could just as easily download them into
+ *			 * files using g_file_create() and g_output_stream_splice(), rather than create GdkPixbuf<!-- -->s directly from them.
+ *			 * Note that this is a blocking operation. *<!-- -->/
+ *			download_stream = gdata_media_thumbnail_download (thumbnail, GDATA_SERVICE (service), NULL, NULL);
+ *			pixbuf = gdk_pixbuf_new_from_stream (G_INPUT_STREAM (download_stream), NULL, NULL);
+ *			g_object_unref (download_stream);
+ *			/<!-- -->* ... *<!-- -->/
+ *			g_object_unref (pixbuf);
+ *		}
+ *
+ *		/<!-- -->* Do something worthwhile with your album data *<!-- -->/
+ *	}
+ *
+ *	g_object_unref (album_feed);
+ * 	</programlisting>
+ * </example>
+ *
  * Since: 0.4.0
  **/
 
@@ -50,6 +100,7 @@
 #include "gdata-picasaweb-enums.h"
 #include "georss/gdata-georss-where.h"
 
+static GObject *gdata_picasaweb_album_constructor (GType type, guint n_construct_params, GObjectConstructParam *construct_params);
 static void gdata_picasaweb_album_dispose (GObject *object);
 static void gdata_picasaweb_album_finalize (GObject *object);
 static void gdata_picasaweb_album_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec);
@@ -62,10 +113,10 @@ struct _GDataPicasaWebAlbumPrivate {
 	gchar *album_id;
 	gchar *user;
 	gchar *nickname;
-	GTimeVal edited;
+	gint64 edited;
 	gchar *location;
 	GDataPicasaWebVisibility visibility;
-	GTimeVal timestamp;
+	gint64 timestamp; /* in milliseconds! */
 	guint num_photos;
 	guint num_photos_remaining;
 	glong bytes_used;
@@ -97,16 +148,17 @@ enum {
 };
 
 G_DEFINE_TYPE (GDataPicasaWebAlbum, gdata_picasaweb_album, GDATA_TYPE_ENTRY)
-#define GDATA_PICASAWEB_ALBUM_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GDATA_TYPE_PICASAWEB_ALBUM, GDataPicasaWebAlbumPrivate))
 
 static void
 gdata_picasaweb_album_class_init (GDataPicasaWebAlbumClass *klass)
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 	GDataParsableClass *parsable_class = GDATA_PARSABLE_CLASS (klass);
+	GDataEntryClass *entry_class = GDATA_ENTRY_CLASS (klass);
 
 	g_type_class_add_private (klass, sizeof (GDataPicasaWebAlbumPrivate));
 
+	gobject_class->constructor = gdata_picasaweb_album_constructor;
 	gobject_class->get_property = gdata_picasaweb_album_get_property;
 	gobject_class->set_property = gdata_picasaweb_album_set_property;
 	gobject_class->dispose = gdata_picasaweb_album_dispose;
@@ -116,6 +168,8 @@ gdata_picasaweb_album_class_init (GDataPicasaWebAlbumClass *klass)
 	parsable_class->get_xml = get_xml;
 	parsable_class->get_namespaces = get_namespaces;
 
+	entry_class->kind_term = "http://schemas.google.com/photos/2007#album";
+
 	/**
 	 * GDataPicasaWebAlbum:album-id
 	 *
@@ -126,13 +180,13 @@ gdata_picasaweb_album_class_init (GDataPicasaWebAlbumClass *klass)
 	 * For more information, see the <ulink type="http" url="http://code.google.com/apis/picasaweb/reference.html#gphoto_id">
 	 * gphoto specification</ulink>.
 	 *
-	 * Since: 0.6.4
+	 * Since: 0.7.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_ALBUM_ID,
-					 g_param_spec_string ("album-id",
-							      "Album ID", "The ID of the album.",
-							      NULL,
-							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("album-id",
+	                                                      "Album ID", "The ID of the album.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataPicasaWebAlbum:user
@@ -145,10 +199,10 @@ gdata_picasaweb_album_class_init (GDataPicasaWebAlbumClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_USER,
-					 g_param_spec_string ("user",
-							      "User", "The username of the album owner.",
-							      NULL,
-							      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("user",
+	                                                      "User", "The username of the album owner.",
+	                                                      NULL,
+	                                                      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataPicasaWebAlbum:nickname
@@ -161,10 +215,10 @@ gdata_picasaweb_album_class_init (GDataPicasaWebAlbumClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_NICKNAME,
-					 g_param_spec_string ("nickname",
-							      "Nickname", "The user's nickname.",
-							      NULL,
-							      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("nickname",
+	                                                      "Nickname", "The user's nickname.",
+	                                                      NULL,
+	                                                      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataPicasaWebAlbum:edited
@@ -177,10 +231,10 @@ gdata_picasaweb_album_class_init (GDataPicasaWebAlbumClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_EDITED,
-					 g_param_spec_boxed ("edited",
-							     "Edited", "The time this album was last edited.",
-							     GDATA_TYPE_G_TIME_VAL,
-							     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_int64 ("edited",
+	                                                     "Edited", "The time this album was last edited.",
+	                                                     -1, G_MAXINT64, -1,
+	                                                     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataPicasaWebAlbum:location
@@ -193,10 +247,10 @@ gdata_picasaweb_album_class_init (GDataPicasaWebAlbumClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_LOCATION,
-					 g_param_spec_string ("location",
-							      "Location", "The user-specified location associated with the album.",
-							      NULL,
-							      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("location",
+	                                                      "Location", "The user-specified location associated with the album.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataPicasaWebAlbum:visibility
@@ -209,15 +263,15 @@ gdata_picasaweb_album_class_init (GDataPicasaWebAlbumClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_VISIBILITY,
-					 g_param_spec_enum ("visibility",
-							    "Visibility", "The visibility (or access rights) of the album.",
-							    GDATA_TYPE_PICASAWEB_VISIBILITY, GDATA_PICASAWEB_PUBLIC,
-							    G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_enum ("visibility",
+	                                                    "Visibility", "The visibility (or access rights) of the album.",
+	                                                    GDATA_TYPE_PICASAWEB_VISIBILITY, GDATA_PICASAWEB_PRIVATE,
+	                                                    G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataPicasaWebAlbum:timestamp
 	 *
-	 * The timestamp of when the album occurred, settable by the user.
+	 * The timestamp of when the album occurred, settable by the user. This a UNIX timestamp in milliseconds (not seconds) since the epoch.
 	 *
 	 * For more information, see the <ulink type="http" url="http://code.google.com/apis/picasaweb/reference.html#gphoto_timestamp">
 	 * gphoto specification</ulink>.
@@ -225,10 +279,10 @@ gdata_picasaweb_album_class_init (GDataPicasaWebAlbumClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_TIMESTAMP,
-					 g_param_spec_boxed ("timestamp",
-							     "Timestamp", "The timestamp of when the album occurred, settable by the user.",
-							     GDATA_TYPE_G_TIME_VAL,
-							     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_int64 ("timestamp",
+	                                                     "Timestamp", "The timestamp of when the album occurred, settable by the user.",
+	                                                     -1, G_MAXINT64, -1,
+	                                                     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/* TODO: Change to photo-count? */
 	/**
@@ -242,10 +296,10 @@ gdata_picasaweb_album_class_init (GDataPicasaWebAlbumClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_NUM_PHOTOS,
-					 g_param_spec_uint ("num-photos",
-							    "Number of photos", "The number of photos and videos in the album.",
-							    0, G_MAXUINT, 0,
-							    G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_uint ("num-photos",
+	                                                    "Number of photos", "The number of photos and videos in the album.",
+	                                                    0, G_MAXUINT, 0,
+	                                                    G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 	/* TODO: Change to remaining-photos-count? */
 	/**
@@ -260,11 +314,10 @@ gdata_picasaweb_album_class_init (GDataPicasaWebAlbumClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_NUM_PHOTOS_REMAINING,
-					 g_param_spec_uint ("num-photos-remaining",
-							    "Number of photo spaces remaining", "The number of photos and videos that can still be"
-							    " uploaded to this album.",
-							    0, G_MAXUINT, 0,
-							    G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_uint ("num-photos-remaining",
+	                                                    "Number of photo spaces remaining", "The number of files spaces still free for uploads.",
+	                                                    0, G_MAXUINT, 0,
+	                                                    G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataPicasaWebAlbum:bytes-used:
@@ -278,10 +331,10 @@ gdata_picasaweb_album_class_init (GDataPicasaWebAlbumClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_BYTES_USED,
-					 g_param_spec_long ("bytes-used",
-							    "Number of bytes used", "The number of bytes consumed by this album and its contents.",
-							    -1, G_MAXLONG, -1,
-							    G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_long ("bytes-used",
+	                                                    "Number of bytes used", "The number of bytes consumed by this album and its contents.",
+	                                                    -1, G_MAXLONG, -1,
+	                                                    G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataPicasaWebAlbum:commenting-enabled:
@@ -291,10 +344,10 @@ gdata_picasaweb_album_class_init (GDataPicasaWebAlbumClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_IS_COMMENTING_ENABLED,
-					 g_param_spec_boolean ("is-commenting-enabled",
-							       "Commenting enabled?", "Whether commenting is enabled for this album.",
-							       FALSE,
-							       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_boolean ("is-commenting-enabled",
+	                                                       "Commenting enabled?", "Whether commenting is enabled for this album.",
+	                                                       FALSE,
+	                                                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataPicasaWebAlbum:comment-count:
@@ -307,15 +360,15 @@ gdata_picasaweb_album_class_init (GDataPicasaWebAlbumClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_COMMENT_COUNT,
-					 g_param_spec_uint ("comment-count",
-							    "Comment count", "The number of comments on the album.",
-							    0, G_MAXUINT, 0,
-							    G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_uint ("comment-count",
+	                                                    "Comment count", "The number of comments on the album.",
+	                                                    0, G_MAXUINT, 0,
+	                                                    G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataPicasaWebAlbum:tags:
 	 *
-	 * A comma-separated list of tags associated with the album; all the tags associated with the individual photos in the album.
+	 * A %NULL-terminated array of tags associated with the album; all the tags associated with the individual photos in the album.
 	 *
 	 * For more information, see the <ulink type="http" url="http://code.google.com/apis/picasaweb/reference.html#media_keywords">
 	 * Media RSS specification</ulink>.
@@ -323,10 +376,10 @@ gdata_picasaweb_album_class_init (GDataPicasaWebAlbumClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_TAGS,
-					 g_param_spec_string ("tags",
-							      "Tags", "A comma-separated list of tags associated with the album",
-							      NULL,
-							      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_boxed ("tags",
+	                                                     "Tags", "A NULL-terminated array of tags associated with the album",
+	                                                     G_TYPE_STRV,
+	                                                     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataPicasaWebAlbum:latitude:
@@ -340,10 +393,10 @@ gdata_picasaweb_album_class_init (GDataPicasaWebAlbumClass *klass)
 	 * Since: 0.5.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_LATITUDE,
-					 g_param_spec_double ("latitude",
-							      "Latitude", "The location as a latitude coordinate associated with this album.",
-							      -90.0, 90.0, 0.0,
-							      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_double ("latitude",
+	                                                      "Latitude", "The location as a latitude coordinate associated with this album.",
+	                                                      -90.0, 90.0, 0.0,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataPicasaWebAlbum:longitude:
@@ -357,10 +410,10 @@ gdata_picasaweb_album_class_init (GDataPicasaWebAlbumClass *klass)
 	 * Since: 0.5.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_LONGITUDE,
-					 g_param_spec_double ("longitude",
-							      "Longitude", "The location as a longitude coordinate associated with this album.",
-							      -180.0, 180.0, 0.0,
-							      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_double ("longitude",
+	                                                      "Longitude", "The location as a longitude coordinate associated with this album.",
+	                                                      -180.0, 180.0, 0.0,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -427,11 +480,12 @@ gdata_picasaweb_album_init (GDataPicasaWebAlbum *self)
 	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GDATA_TYPE_PICASAWEB_ALBUM, GDataPicasaWebAlbumPrivate);
 	self->priv->media_group = g_object_new (GDATA_TYPE_MEDIA_GROUP, NULL);
 	self->priv->georss_where = g_object_new (GDATA_TYPE_GEORSS_WHERE, NULL);
-	self->priv->visibility = GDATA_PICASAWEB_PRIVATE;
+	self->priv->edited = -1;
+	self->priv->timestamp = -1;
 
-	/* Initialise the timestamp and edited properties to the current time (bgo#599140) */
-	g_get_current_time (&(self->priv->timestamp));
-	g_get_current_time (&(self->priv->edited));
+	/* Set the initial visibility */
+	self->priv->visibility = GDATA_PICASAWEB_PRIVATE;
+	gdata_entry_set_rights (GDATA_ENTRY (self), "private");
 
 	/* Connect to the notify::title signal from GDataEntry so our media:group title can be kept in sync
 	 * (the title of an album is duplicated in atom:title and media:group/media:title) */
@@ -445,10 +499,32 @@ gdata_picasaweb_album_init (GDataPicasaWebAlbum *self)
 	g_signal_connect (self, "notify::visibility", G_CALLBACK (notify_visibility_cb), NULL);
 }
 
+static GObject *
+gdata_picasaweb_album_constructor (GType type, guint n_construct_params, GObjectConstructParam *construct_params)
+{
+	GObject *object;
+
+	/* Chain up to the parent class */
+	object = G_OBJECT_CLASS (gdata_picasaweb_album_parent_class)->constructor (type, n_construct_params, construct_params);
+
+	if (_gdata_parsable_is_constructed_from_xml (GDATA_PARSABLE (object)) == FALSE) {
+		GDataPicasaWebAlbumPrivate *priv = GDATA_PICASAWEB_ALBUM (object)->priv;
+		GTimeVal time_val;
+
+		/* Set the edited and timestamp properties to the current time (creation time). bgo#599140
+		 * We don't do this in *_init() since that would cause setting it from parse_xml() to fail (duplicate element). */
+		g_get_current_time (&time_val);
+		priv->timestamp = time_val.tv_sec * 1000;
+		priv->edited = time_val.tv_sec;
+	}
+
+	return object;
+}
+
 static void
 gdata_picasaweb_album_dispose (GObject *object)
 {
-	GDataPicasaWebAlbumPrivate *priv = GDATA_PICASAWEB_ALBUM_GET_PRIVATE (object);
+	GDataPicasaWebAlbumPrivate *priv = GDATA_PICASAWEB_ALBUM (object)->priv;
 
 	if (priv->media_group != NULL)
 		g_object_unref (priv->media_group);
@@ -465,7 +541,7 @@ gdata_picasaweb_album_dispose (GObject *object)
 static void
 gdata_picasaweb_album_finalize (GObject *object)
 {
-	GDataPicasaWebAlbumPrivate *priv = GDATA_PICASAWEB_ALBUM_GET_PRIVATE (object);
+	GDataPicasaWebAlbumPrivate *priv = GDATA_PICASAWEB_ALBUM (object)->priv;
 
 	g_free (priv->album_id);
 	g_free (priv->user);
@@ -479,7 +555,7 @@ gdata_picasaweb_album_finalize (GObject *object)
 static void
 gdata_picasaweb_album_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec)
 {
-	GDataPicasaWebAlbumPrivate *priv = GDATA_PICASAWEB_ALBUM_GET_PRIVATE (object);
+	GDataPicasaWebAlbumPrivate *priv = GDATA_PICASAWEB_ALBUM (object)->priv;
 
 	switch (property_id) {
 		case PROP_ALBUM_ID:
@@ -492,7 +568,7 @@ gdata_picasaweb_album_get_property (GObject *object, guint property_id, GValue *
 			g_value_set_string (value, priv->nickname);
 			break;
 		case PROP_EDITED:
-			g_value_set_boxed (value, &(priv->edited));
+			g_value_set_int64 (value, priv->edited);
 			break;
 		case PROP_LOCATION:
 			g_value_set_string (value, priv->location);
@@ -501,7 +577,7 @@ gdata_picasaweb_album_get_property (GObject *object, guint property_id, GValue *
 			g_value_set_enum (value, priv->visibility);
 			break;
 		case PROP_TIMESTAMP:
-			g_value_set_boxed (value, &(priv->timestamp));
+			g_value_set_int64 (value, priv->timestamp);
 			break;
 		case PROP_NUM_PHOTOS:
 			g_value_set_uint (value, priv->num_photos);
@@ -519,7 +595,7 @@ gdata_picasaweb_album_get_property (GObject *object, guint property_id, GValue *
 			g_value_set_uint (value, priv->comment_count);
 			break;
 		case PROP_TAGS:
-			g_value_set_string (value, gdata_media_group_get_keywords (priv->media_group));
+			g_value_set_boxed (value, gdata_media_group_get_keywords (priv->media_group));
 			break;
 		case PROP_LATITUDE:
 			g_value_set_double (value, gdata_georss_where_get_latitude (priv->georss_where));
@@ -552,13 +628,13 @@ gdata_picasaweb_album_set_property (GObject *object, guint property_id, const GV
 			gdata_picasaweb_album_set_visibility (self, g_value_get_enum (value));
 			break;
 		case PROP_TIMESTAMP:
-			gdata_picasaweb_album_set_timestamp (self, g_value_get_boxed (value));
+			gdata_picasaweb_album_set_timestamp (self, g_value_get_int64 (value));
 			break;
 		case PROP_IS_COMMENTING_ENABLED:
 			gdata_picasaweb_album_set_is_commenting_enabled (self, g_value_get_boolean (value));
 			break;
 		case PROP_TAGS:
-			gdata_picasaweb_album_set_tags (self, g_value_get_string (value));
+			gdata_picasaweb_album_set_tags (self, g_value_get_boxed (value));
 			break;
 		case PROP_LATITUDE:
 			gdata_picasaweb_album_set_coordinates (self, g_value_get_double (value),
@@ -578,94 +654,52 @@ gdata_picasaweb_album_set_property (GObject *object, guint property_id, const GV
 static gboolean
 parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_data, GError **error)
 {
+	gboolean success;
 	GDataPicasaWebAlbum *self = GDATA_PICASAWEB_ALBUM (parsable);
 
-	if (gdata_parser_is_namespace (node, "http://search.yahoo.com/mrss/") == TRUE &&
-	    xmlStrcmp (node->name, (xmlChar*) "group") == 0) {
-		/* media:group */
-		GDataMediaGroup *group = GDATA_MEDIA_GROUP (_gdata_parsable_new_from_xml_node (GDATA_TYPE_MEDIA_GROUP, doc, node, NULL, error));
-		if (group == NULL)
-			return FALSE;
-
-		if (self->priv->media_group != NULL)
-			/* We should really error here, but we can't, as priv->media_group has to be pre-populated
-			 * in order for things like gdata_picasaweb_album_get_tags() to work. */
-			g_object_unref (self->priv->media_group);
-
-		self->priv->media_group = group;
+	/* TODO: media:group should also be P_NO_DUPES, but we can't, as priv->media_group has to be pre-populated
+	 * in order for things like gdata_picasaweb_album_get_tags() to work. */
+	if (gdata_parser_is_namespace (node, "http://www.w3.org/2007/app") == TRUE &&
+	    gdata_parser_int64_from_element (node, "edited", P_REQUIRED | P_NO_DUPES, &(self->priv->edited), &success, error) == TRUE) {
+		return success;
+	} else if (gdata_parser_is_namespace (node, "http://search.yahoo.com/mrss/") == TRUE &&
+	           gdata_parser_object_from_element (node, "group", P_REQUIRED, GDATA_TYPE_MEDIA_GROUP,
+	                                             &(self->priv->media_group), &success, error) == TRUE) {
+		return success;
 	} else if (gdata_parser_is_namespace (node, "http://www.georss.org/georss") == TRUE &&
-	           xmlStrcmp (node->name, (xmlChar*) "where") == 0) {
-		/* georss:where */
-		GDataGeoRSSWhere *where = GDATA_GEORSS_WHERE (_gdata_parsable_new_from_xml_node (GDATA_TYPE_GEORSS_WHERE, doc, node, NULL, error));
-		if (where == NULL)
-			return FALSE;
-
-		if (self->priv->georss_where != NULL)
-			g_object_unref (self->priv->georss_where);
-
-		self->priv->georss_where = where;
-	} else if (gdata_parser_is_namespace (node, "http://www.w3.org/2007/app") == TRUE &&
-	           xmlStrcmp (node->name, (xmlChar*) "edited") == 0) {
-		/* app:edited */
-		xmlChar *edited = xmlNodeListGetString (doc, node->children, TRUE);
-		if (g_time_val_from_iso8601 ((gchar*) edited, &(self->priv->edited)) == FALSE) {
-			/* Error */
-			gdata_parser_error_not_iso8601_format (node, (gchar*) edited, error);
-			xmlFree (edited);
-			return FALSE;
-		}
-		xmlFree (edited);
+	           gdata_parser_object_from_element (node, "where", P_REQUIRED, GDATA_TYPE_GEORSS_WHERE,
+	                                             &(self->priv->georss_where), &success, error) == TRUE) {
+		return success;
 	} else if (gdata_parser_is_namespace (node, "http://schemas.google.com/photos/2007") == TRUE) {
-		if (xmlStrcmp (node->name, (xmlChar*) "user") == 0) {
-			/* gphoto:user */
-			xmlChar *user = xmlNodeListGetString (doc, node->children, TRUE);
-			if (user == NULL || *user == '\0')
-				return gdata_parser_error_required_content_missing (node, error);
-			g_free (self->priv->user);
-			self->priv->user = (gchar*) user;
-		} else if (xmlStrcmp (node->name, (xmlChar*) "nickname") == 0) {
-			/* gphoto:nickname */
-			xmlChar *nickname = xmlNodeListGetString (doc, node->children, TRUE);
-			if (nickname == NULL || *nickname == '\0')
-				return gdata_parser_error_required_content_missing (node, error);
-			g_free (self->priv->nickname);
-			self->priv->nickname = (gchar*) nickname;
-		} else if (xmlStrcmp (node->name, (xmlChar*) "location") == 0) {
-			/* gphoto:location */
-			g_free (self->priv->location);
-			self->priv->location = (gchar*) xmlNodeListGetString (doc, node->children, TRUE);
-		} else if (xmlStrcmp (node->name, (xmlChar*) "id") == 0) {
-			/* gphoto:id */
-			xmlChar *id = xmlNodeListGetString (doc, node->children, TRUE);
-			if (id == NULL || *id == '\0')
-				return gdata_parser_error_required_content_missing (node, error);
-			g_free (self->priv->album_id);
-			self->priv->album_id = (gchar*) id;
+		if (gdata_parser_string_from_element (node, "user", P_REQUIRED | P_NON_EMPTY, &(self->priv->user), &success, error) == TRUE ||
+		    gdata_parser_string_from_element (node, "nickname", P_REQUIRED | P_NON_EMPTY, &(self->priv->nickname), &success, error) == TRUE ||
+		    gdata_parser_string_from_element (node, "location", P_NONE, &(self->priv->location), &success, error) == TRUE ||
+		    gdata_parser_string_from_element (node, "id", P_REQUIRED | P_NON_EMPTY | P_NO_DUPES,
+		                                      &(self->priv->album_id), &success, error) == TRUE) {
+			return success;
 		} else if (xmlStrcmp (node->name, (xmlChar*) "access") == 0) {
 			/* gphoto:access */
-			xmlChar *access = xmlNodeListGetString (doc, node->children, TRUE);
-			if (xmlStrcmp (access, (xmlChar*) "public") == 0) {
+			xmlChar *access_level = xmlNodeListGetString (doc, node->children, TRUE);
+			if (xmlStrcmp (access_level, (xmlChar*) "public") == 0) {
 				gdata_picasaweb_album_set_visibility (self, GDATA_PICASAWEB_PUBLIC);
-			} else if (xmlStrcmp (access, (xmlChar*) "private") == 0) {
+			} else if (xmlStrcmp (access_level, (xmlChar*) "private") == 0) {
 				gdata_picasaweb_album_set_visibility (self, GDATA_PICASAWEB_PRIVATE);
 			} else {
-				gdata_parser_error_unknown_content (node, (gchar*) access, error);
+				gdata_parser_error_unknown_content (node, (gchar*) access_level, error);
+				xmlFree (access_level);
+				return FALSE;
 			}
-			xmlFree (access);
+			xmlFree (access_level);
 		} else if (xmlStrcmp (node->name, (xmlChar*) "timestamp") == 0) {
 			/* gphoto:timestamp */
 			xmlChar *timestamp_str;
 			guint64 milliseconds;
-			GTimeVal timestamp;
 
 			timestamp_str = xmlNodeListGetString (doc, node->children, TRUE);
 			milliseconds = g_ascii_strtoull ((gchar*) timestamp_str, NULL, 10);
 			xmlFree (timestamp_str);
 
-			timestamp.tv_sec = (glong) (milliseconds / 1000);
-			timestamp.tv_usec = (glong) ((milliseconds % 1000) * 1000);
-
-			gdata_picasaweb_album_set_timestamp (self, &timestamp);
+			gdata_picasaweb_album_set_timestamp (self, (gint64) milliseconds);
 		} else if (xmlStrcmp (node->name, (xmlChar*) "numphotos") == 0) {
 			/* gphoto:numphotos */
 			xmlChar *num_photos = xmlNodeListGetString (doc, node->children, TRUE);
@@ -704,7 +738,8 @@ parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_da
 				return gdata_parser_error_required_content_missing (node, error);
 			}
 
-			gdata_picasaweb_album_set_is_commenting_enabled (self, (xmlStrcmp (commenting_enabled, (xmlChar*) "true") == 0) ? TRUE : FALSE);
+			gdata_picasaweb_album_set_is_commenting_enabled (self,
+			                                                 (xmlStrcmp (commenting_enabled, (xmlChar*) "true") == 0) ? TRUE : FALSE);
 			xmlFree (commenting_enabled);
 		} else if (xmlStrcmp (node->name, (xmlChar*) "commentCount") == 0) {
 			/* gphoto:commentCount */
@@ -736,7 +771,7 @@ get_xml (GDataParsable *parsable, GString *xml_string)
 
 	/* Add all the album-specific XML */
 	if (priv->album_id != NULL)
-		g_string_append_printf (xml_string, "<gphoto:id>%s</gphoto:id>", priv->album_id);
+		gdata_parser_string_append_escaped (xml_string, "<gphoto:id>", priv->album_id, "</gphoto:id>");
 
 	if (priv->location != NULL)
 		gdata_parser_string_append_escaped (xml_string, "<gphoto:location>", priv->location, "</gphoto:location>");
@@ -752,10 +787,9 @@ get_xml (GDataParsable *parsable, GString *xml_string)
 			g_assert_not_reached ();
 	}
 
-	if (priv->timestamp.tv_sec != 0 || priv->timestamp.tv_usec != 0) {
+	if (priv->timestamp != -1) {
 		/* in milliseconds */
-		g_string_append_printf (xml_string, "<gphoto:timestamp>%" G_GUINT64_FORMAT "</gphoto:timestamp>",
-					((guint64) priv->timestamp.tv_sec) * 1000 + priv->timestamp.tv_usec / 1000);
+		g_string_append_printf (xml_string, "<gphoto:timestamp>%" G_GINT64_FORMAT "</gphoto:timestamp>", priv->timestamp);
 	}
 
 	if (priv->is_commenting_enabled == FALSE)
@@ -775,8 +809,6 @@ get_xml (GDataParsable *parsable, GString *xml_string)
 	/* TODO:
 	 * - Finish supporting all tags
 	 * - Check all tags here are valid for insertions and updates
-	 * - Check things are escaped (or not) as appropriate
-	 * - add GML support
 	 */
 }
 
@@ -839,7 +871,7 @@ gdata_picasaweb_album_new (const gchar *id)
  *
  * Return value: the album's ID
  *
- * Since: 0.6.4
+ * Since: 0.7.0
  **/
 const gchar *
 gdata_picasaweb_album_get_id (GDataPicasaWebAlbum *self)
@@ -885,19 +917,18 @@ gdata_picasaweb_album_get_nickname (GDataPicasaWebAlbum *self)
 /**
  * gdata_picasaweb_album_get_edited:
  * @self: a #GDataPicasaWebAlbum
- * @edited: (out caller-allocates): a #GTimeVal
  *
- * Gets the #GDataPicasaWebAlbum:edited property and puts it in @edited. If the property is unset,
- * both fields in the #GTimeVal will be set to <code class="literal">0</code>.
+ * Gets the #GDataPicasaWebAlbum:edited property. If the property is unset, <code class="literal">-1</code> will be returned.
+ *
+ * Return value: the UNIX timestamp for the time the album was last edited, or <code class="literal">-1</code>
  *
  * Since: 0.4.0
  **/
-void
-gdata_picasaweb_album_get_edited (GDataPicasaWebAlbum *self, GTimeVal *edited)
+gint64
+gdata_picasaweb_album_get_edited (GDataPicasaWebAlbum *self)
 {
-	g_return_if_fail (GDATA_IS_PICASAWEB_ALBUM (self));
-	g_return_if_fail (edited != NULL);
-	*edited = self->priv->edited;
+	g_return_val_if_fail (GDATA_IS_PICASAWEB_ALBUM (self), -1);
+	return self->priv->edited;
 }
 
 /**
@@ -976,40 +1007,40 @@ gdata_picasaweb_album_set_visibility (GDataPicasaWebAlbum *self, GDataPicasaWebV
 /**
  * gdata_picasaweb_album_get_timestamp:
  * @self: a #GDataPicasaWebAlbum
- * @timestamp: (out caller-allocates): a #GTimeVal
  *
- * Gets the #GDataPicasaWebAlbum:timestamp property and puts it in @timestamp. If the property is unset,
- * both fields in the #GTimeVal will be set to <code class="literal">0</code>.
+ * Gets the #GDataPicasaWebAlbum:timestamp property. This value usually holds either the date that best corresponds to the album of photos, or to the
+ * day it was uploaded. It's a UNIX timestamp in milliseconds (not seconds) since the epoch. If the property is unset, <code class="literal">-1</code>
+ * will be returned.
+ *
+ * Return value: the UNIX timestamp for the timestamp property in milliseconds, or <code class="literal">-1</code>
  *
  * Since: 0.4.0
  **/
-void
-gdata_picasaweb_album_get_timestamp (GDataPicasaWebAlbum *self, GTimeVal *timestamp)
+gint64
+gdata_picasaweb_album_get_timestamp (GDataPicasaWebAlbum *self)
 {
-	g_return_if_fail (GDATA_IS_PICASAWEB_ALBUM (self));
-	g_return_if_fail (timestamp != NULL);
-	*timestamp = self->priv->timestamp;
+	g_return_val_if_fail (GDATA_IS_PICASAWEB_ALBUM (self), -1);
+	return self->priv->timestamp;
 }
 
 /**
  * gdata_picasaweb_album_set_timestamp:
  * @self: a #GDataPicasaWebAlbum
- * @timestamp: (allow-none): a #GTimeVal, or %NULL
+ * @timestamp: a UNIX timestamp, or <code class="literal">-1</code>
  *
- * Sets the #GDataPicasaWebAlbum:timestamp property from values supplied by @timestamp.
+ * Sets the #GDataPicasaWebAlbum:timestamp property from @timestamp. This should be a UNIX timestamp in milliseconds (not seconds) since the epoch.
  *
- * Set @timestamp to %NULL to unset the property.
+ * Set @timestamp to <code class="literal">-1</code> to unset the property.
  *
  * Since: 0.4.0
  **/
 void
-gdata_picasaweb_album_set_timestamp (GDataPicasaWebAlbum *self, GTimeVal *timestamp)
+gdata_picasaweb_album_set_timestamp (GDataPicasaWebAlbum *self, gint64 timestamp)
 {
 	g_return_if_fail (GDATA_IS_PICASAWEB_ALBUM (self));
-	if (timestamp == NULL)
-		self->priv->timestamp.tv_sec = self->priv->timestamp.tv_usec = 0;
-	else
-		self->priv->timestamp = *timestamp;
+	g_return_if_fail (timestamp >= -1);
+
+	self->priv->timestamp = timestamp;
 	g_object_notify (G_OBJECT (self), "timestamp");
 }
 
@@ -1122,11 +1153,11 @@ gdata_picasaweb_album_get_comment_count (GDataPicasaWebAlbum *self)
  *
  * Gets the #GDataPicasaWebAlbum:tags property.
  *
- * Return value: a comma-separated list of tags associated with all the photos in the album, or %NULL
+ * Return value: (array zero-terminated=1) (transfer none): a %NULL-terminated array of tags associated with all the photos in the album, or %NULL
  *
  * Since: 0.4.0
  **/
-const gchar *
+const gchar * const *
 gdata_picasaweb_album_get_tags (GDataPicasaWebAlbum *self)
 {
 	g_return_val_if_fail (GDATA_IS_PICASAWEB_ALBUM (self), NULL);
@@ -1136,7 +1167,7 @@ gdata_picasaweb_album_get_tags (GDataPicasaWebAlbum *self)
 /**
  * gdata_picasaweb_album_set_tags:
  * @self: a #GDataPicasaWebAlbum
- * @tags: (allow-none): the new comma-separated list of tags, or %NULL
+ * @tags: (array zero-terminated=1) (allow-none): the new %NULL-terminated array of tags, or %NULL
  *
  * Sets the #GDataPicasaWebAlbum:tags property to @tags.
  *
@@ -1145,7 +1176,7 @@ gdata_picasaweb_album_get_tags (GDataPicasaWebAlbum *self)
  * Since: 0.4.0
  **/
 void
-gdata_picasaweb_album_set_tags (GDataPicasaWebAlbum *self, const gchar *tags)
+gdata_picasaweb_album_set_tags (GDataPicasaWebAlbum *self, const gchar * const *tags)
 {
 	g_return_if_fail (GDATA_IS_PICASAWEB_ALBUM (self));
 
